@@ -16,8 +16,9 @@ import datetime
 import mimetypes
 import shutil
 from collections.abc import KeysView, MutableMapping
+from contextlib import ExitStack
 from decimal import Decimal
-from io import BytesIO
+from io import BytesIO, RawIOBase
 from pathlib import Path
 from subprocess import run
 from tempfile import NamedTemporaryFile
@@ -26,7 +27,7 @@ from warnings import warn
 
 from . import Array, Dictionary, Name, Object, Page, Pdf, Stream
 from ._augments import augment_override_cpp, augments
-from ._qpdf import (
+from ._core import (
     AccessMode,
     AttachedFile,
     AttachedFileSpec,
@@ -40,6 +41,7 @@ from ._qpdf import (
     Token,
     _ObjectMapping,
 )
+from ._io import atomic_overwrite, check_different_files, check_stream_is_usable
 from .models import Encryption, EncryptionInfo, Outline, PdfMetadata, Permissions
 from .models.metadata import decode_pdf_date, encode_pdf_date
 
@@ -175,8 +177,7 @@ class Extend_Object:
         decode_parms: Dictionary | Array | None = None,
         type_check: bool = True,
     ):  # pylint: disable=redefined-builtin
-        """
-        Replace stream object's data with new (possibly compressed) `data`.
+        """Replace stream object's data with new (possibly compressed) `data`.
 
         `filter` and `decode_parms` describe any compression that is already
         present on the input `data`. For example, if your data is already
@@ -220,10 +221,10 @@ class Extend_Pdf:
     def _repr_mimebundle_(
         self, include=None, exclude=None
     ):  # pylint: disable=unused-argument
-        """
-        Present options to IPython or Jupyter for rich display of this object.
+        """Present options to IPython or Jupyter for rich display of this object.
 
-        See https://ipython.readthedocs.io/en/stable/config/integrating.html#rich-display
+        See:
+        https://ipython.readthedocs.io/en/stable/config/integrating.html#rich-display
         """
         bio = BytesIO()
         self.save(bio)
@@ -234,8 +235,7 @@ class Extend_Pdf:
 
     @property
     def docinfo(self) -> Dictionary:
-        """
-        Access the (deprecated) document information dictionary.
+        """Access the (deprecated) document information dictionary.
 
         The document information dictionary is a brief metadata record that can
         store some information about the origin of a PDF. It is deprecated and
@@ -257,7 +257,7 @@ class Extend_Pdf:
         ``del pdf.docinfo``. Note that accessing the property after deleting it
         will re-create with a new, empty dictionary.
 
-        .. versionchanged: 2.4
+        .. versionchanged:: 2.4
             Added support for ``del pdf.docinfo``.
         """
         if Name.Info not in self.trailer:
@@ -283,8 +283,7 @@ class Extend_Pdf:
         update_docinfo: bool = True,
         strict: bool = False,
     ) -> PdfMetadata:
-        """
-        Open the PDF's XMP metadata for editing.
+        """Open the PDF's XMP metadata for editing.
 
         There is no ``.close()`` function on the metadata object, since this is
         intended to be used inside a ``with`` block only.
@@ -331,8 +330,7 @@ class Extend_Pdf:
         )
 
     def open_outline(self, max_depth: int = 15, strict: bool = False) -> Outline:
-        """
-        Open the PDF outline ("bookmarks") for editing.
+        """Open the PDF outline ("bookmarks") for editing.
 
         Recommend for use in a ``with`` block. Changes are committed to the
         PDF when the block exits. (The ``Pdf`` must still be opened.)
@@ -362,8 +360,7 @@ class Extend_Pdf:
         return Outline(self, max_depth=max_depth, strict=strict)
 
     def make_stream(self, data: bytes, d=None, **kwargs) -> Stream:
-        """
-        Create a new pikepdf.Stream object that is attached to this PDF.
+        """Create a new pikepdf.Stream object that is attached to this PDF.
 
         See:
             :meth:`pikepdf.Stream.__new__`
@@ -374,8 +371,7 @@ class Extend_Pdf:
     def add_blank_page(
         self, *, page_size: tuple[Numeric, Numeric] = (612.0, 792.0)
     ) -> Page:
-        """
-        Add a blank page to this PDF.
+        """Add a blank page to this PDF.
 
         If pages already exist, the page will be added to the end. Pages may be
         reordered using ``Pdf.pages``.
@@ -402,8 +398,7 @@ class Extend_Pdf:
         return Page(page_obj)
 
     def close(self) -> None:
-        """
-        Close a ``Pdf`` object and release resources acquired by pikepdf.
+        """Close a ``Pdf`` object and release resources acquired by pikepdf.
 
         If pikepdf opened the file handle it will close it (e.g. when opened with a file
         path). If the caller opened the file for pikepdf, the caller close the file.
@@ -414,9 +409,9 @@ class Extend_Pdf:
         case for :class:`pikepdf.Stream` but can be true for any object. Do not close
         the `Pdf` object if you might still be accessing content from it.
 
-        When an ``Object`` is copied from one ``Pdf`` to another, the ``Object`` is copied into
-        the destination ``Pdf`` immediately, so after accessing all desired information
-        from the source ``Pdf`` it may be closed.
+        When an ``Object`` is copied from one ``Pdf`` to another, the ``Object`` is
+        copied into the destination ``Pdf`` immediately, so after accessing all desired
+        information from the source ``Pdf`` it may be closed.
 
         .. versionchanged:: 3.0
             In pikepdf 2.x, this function actually worked by resetting to a very short
@@ -434,8 +429,7 @@ class Extend_Pdf:
 
     @property
     def allow(self) -> Permissions:
-        """
-        Report permissions associated with this PDF.
+        """Report permissions associated with this PDF.
 
         By default these permissions will be replicated when the PDF is
         saved. Permissions may also only be changed when a PDF is being saved,
@@ -451,18 +445,39 @@ class Extend_Pdf:
 
     @property
     def encryption(self) -> EncryptionInfo:
-        """
-        Report encryption information for this PDF.
+        """Report encryption information for this PDF.
 
         Encryption settings may only be changed when a PDF is saved.
         """
         return EncryptionInfo(self._encryption_data)
 
     def check(self) -> list[str]:
-        """
-        Check if PDF is well-formed.
+        """Check if PDF is syntactically well-formed.
 
-        Similar to ``qpdf --check``.
+        Similar to ``qpdf --check``, checks for syntax
+        or structural problems in the PDF. This is mainly useful to PDF
+        developers and may not be informative to the average user. PDFs with
+        these problems still render correctly, if PDF viewers are capable of
+        working around the issues they contain. In many cases, pikepdf can
+        also fix the problems.
+
+        An example problem found by this function is a xref table that is
+        missing an object reference. A page dictionary with the wrong type of
+        key, such as a string instead of an array of integers for its mediabox,
+        is not the sort of issue checked for. If this were an XML checker, it
+        would tell you if the XML is well-formed, but could not tell you if
+        the XML is valid XHTML or if it can be rendered as a usable web page.
+
+        This function also attempts to decompress all streams in the PDF.
+        If no JBIG2 decoder is available and JBIG2 images are presented,
+        a warning will occur that JBIG2 cannot be checked.
+
+        This function returns a list of strings describing the issues. The
+        text is subject to change and should not be treated as a stable API.
+
+        Returns:
+            Empty list if no issues were found. List of issues as text strings
+            if issues were found.
         """
 
         class DiscardingParser(StreamParser):
@@ -506,9 +521,9 @@ class Extend_Pdf:
         progress: Callable[[int], None] = None,
         encryption: Encryption | bool | None = None,
         recompress_flate: bool = False,
+        deterministic_id: bool = False,
     ) -> None:
-        """
-        Save all modifications to this :class:`pikepdf.Pdf`.
+        """Save all modifications to this :class:`pikepdf.Pdf`.
 
         Args:
             filename_or_stream: Where to write the output. If a file
@@ -522,8 +537,10 @@ class Extend_Pdf:
 
             static_id: Indicates that the ``/ID`` metadata, normally
                 calculated as a hash of certain PDF contents and metadata
-                including the current time, should instead be generated
-                deterministically. Normally for debugging.
+                including the current time, should instead be set to a static
+                value. Only use this for debugging and testing. Use
+                ``deterministic_id`` if you want to get the same ``/ID`` for
+                the same document contents.
             preserve_pdfa: Ensures that the file is generated in a
                 manner compliant with PDF/A and other stricter variants.
                 This should be True, the default, in most cases.
@@ -552,15 +569,34 @@ class Extend_Pdf:
                 creating the smallest files but requiring PDF 1.5+.
 
             compress_streams: Enables or disables the compression of
-                stream objects in the PDF that are created without specifying
-                any compression setting. Metadata is never compressed.
-                By default this is set to ``True``, and should be except
-                for debugging. Existing streams in the PDF or streams will not
-                be modified. To decompress existing streams, you must set
+                uncompressed stream objects. By default this is set to
+                ``True``, and the only reason to set it to ``False`` is for
+                debugging or inspecting PDF contents.
+
+                When enabled, uncompressed stream objects will be compressed
+                whether they were uncompressed in the PDF when it was opened,
+                or when the user creates new :class:`pikepdf.Stream` objects
+                attached to the PDF. Stream objects can also be created
+                indirectly, such as when content from another PDF is merged
+                into the one being saved.
+
+                Only stream objects that have no compression will be
+                compressed when this object is set. If the object is
+                compressed, compression will be preserved.
+
+                Setting compress_streams=False does not trigger decompression
+                unless decompression is specifically requested by setting
                 both ``compress_streams=False`` and ``stream_decode_level``
                 to the desired decode level (e.g. ``.generalized`` will
                 decompress most non-image content).
 
+                This option does not trigger recompression of existing
+                compressed streams. For that, use ``recompress_flate``.
+
+                The XMP metadata stream object, if present, is never
+                compressed, to facilitate metadata reading by parsers that
+                don't understand the full structure of PDF.
+
             stream_decode_level: Specifies how
                 to encode stream objects. See documentation for
                 :class:`pikepdf.StreamDecodeLevel`.
@@ -597,6 +633,14 @@ class Extend_Pdf:
                 Alternately, an ``Encryption`` object may be provided that
                 sets the parameters for new encryption.
 
+            deterministic_id: Indicates that the ``/ID`` metadata, normally
+                calculated as a hash of certain PDF contents and metadata
+                including the current time, should instead be computed using
+                only deterministic data like the file contents. At a small
+                runtime cost, this enables generation of the same ``/ID`` if
+                the same inputs are converted in the same way multiple times.
+                Does not work for encrypted files.
+
         Raises:
             PdfError
             ForeignObjectError
@@ -621,12 +665,28 @@ class Extend_Pdf:
             coalesces any incremental updates into a single non-incremental
             PDF file when saving.
 
+        .. note::
+            If filename_or_stream is a stream and the process is interrupted during
+            writing, the stream may be left in a corrupt state. It is the
+            responsibility of the caller to manage the stream in this case.
+
         .. versionchanged:: 2.7
             Added *recompress_flate*.
 
         .. versionchanged:: 3.0
             Keyword arguments now mandatory for everything except the first
             argument.
+
+        .. versionchanged:: 8.1
+            If filename_or_stream is a filename and that file exists, the new file
+            is written to a temporary file in the same directory and then moved into
+            place. This prevents the existing destination file from being corrupted
+            if the process is interrupted during writing; previously, corrupting the
+            destination file was possible. If no file exists at the destination, output
+            is written directly to the destination, but the destination will be deleted
+            if errors occur during writing. Prior to 8.1, the file was always written
+            directly to the destination, which could result in a corrupt destination
+            file if the process was interrupted during writing.
         """
         if not filename_or_stream and getattr(self, '_original_filename', None):
             filename_or_stream = self._original_filename
@@ -639,24 +699,39 @@ class Extend_Pdf:
                 "Pdf.new(), you must specify a destination object since there is "
                 "no original filename to save to."
             )
-        self._save(
-            filename_or_stream,
-            static_id=static_id,
-            preserve_pdfa=preserve_pdfa,
-            min_version=min_version,
-            force_version=force_version,
-            fix_metadata_version=fix_metadata_version,
-            compress_streams=compress_streams,
-            stream_decode_level=stream_decode_level,
-            object_stream_mode=object_stream_mode,
-            normalize_content=normalize_content,
-            linearize=linearize,
-            qdf=qdf,
-            progress=progress,
-            encryption=encryption,
-            samefile_check=getattr(self, '_tmp_stream', None) is None,
-            recompress_flate=recompress_flate,
-        )
+        with ExitStack() as stack:
+            if hasattr(filename_or_stream, 'seek'):
+                stream = filename_or_stream
+                check_stream_is_usable(filename_or_stream)
+            else:
+                if not isinstance(filename_or_stream, (str, bytes, Path)):
+                    raise TypeError("expected str, bytes or os.PathLike object")
+                filename = Path(filename_or_stream)
+                if (
+                    not getattr(self, '_tmp_stream', None)
+                    and getattr(self, '_original_filename', None) is not None
+                ):
+                    check_different_files(self._original_filename, filename)
+                stream = stack.enter_context(atomic_overwrite(filename))
+            self._save(
+                stream,
+                static_id=static_id,
+                preserve_pdfa=preserve_pdfa,
+                min_version=min_version,
+                force_version=force_version,
+                fix_metadata_version=fix_metadata_version,
+                compress_streams=compress_streams,
+                stream_decode_level=stream_decode_level,
+                object_stream_mode=object_stream_mode,
+                normalize_content=normalize_content,
+                linearize=linearize,
+                qdf=qdf,
+                progress=progress,
+                encryption=encryption,
+                samefile_check=getattr(self, '_tmp_stream', None) is None,
+                recompress_flate=recompress_flate,
+                deterministic_id=deterministic_id,
+            )
 
     @staticmethod
     def open(
@@ -671,8 +746,7 @@ class Extend_Pdf:
         access_mode: AccessMode = AccessMode.default,
         allow_overwriting_input: bool = False,
     ) -> Pdf:
-        """
-        Open an existing file at *filename_or_stream*.
+        """Open an existing file at *filename_or_stream*.
 
         If *filename_or_stream* is path-like, the file will be opened for reading.
         The file should not be modified by another process while it is open in
@@ -774,8 +848,15 @@ class Extend_Pdf:
                 "expects a filename or opened file-like object. Instead, please use "
                 "Pdf.open(BytesIO(data))."
             )
+        if isinstance(filename_or_stream, int):
+            # Attempted to open with integer file descriptor?
+            # TODO improve error
+            raise TypeError("expected str, bytes or os.PathLike object")
+
+        stream: RawIOBase | None = None
+        closing_stream: bool = False
+        original_filename: Path | None = None
 
-        tmp_stream, original_filename = None, False
         if allow_overwriting_input:
             try:
                 Path(filename_or_stream)
@@ -786,10 +867,25 @@ class Extend_Pdf:
                 ) from error
             original_filename = Path(filename_or_stream)
             with open(original_filename, 'rb') as pdf_file:
-                tmp_stream = BytesIO()
-                shutil.copyfileobj(pdf_file, tmp_stream)
+                stream = BytesIO()
+                shutil.copyfileobj(pdf_file, stream)
+                stream.seek(0)
+            # description = f"memory copy of {original_filename}"
+            description = str(original_filename)
+        elif hasattr(filename_or_stream, 'read') and hasattr(
+            filename_or_stream, 'seek'
+        ):
+            stream = filename_or_stream
+            description = f"stream {stream}"
+        else:
+            stream = open(filename_or_stream, 'rb')
+            original_filename = Path(filename_or_stream)
+            description = str(filename_or_stream)
+            closing_stream = True
+
+        check_stream_is_usable(stream)
         pdf = Pdf._open(
-            tmp_stream or filename_or_stream,
+            stream,
             password=password,
             hex_password=hex_password,
             ignore_xref_streams=ignore_xref_streams,
@@ -797,8 +893,10 @@ class Extend_Pdf:
             attempt_recovery=attempt_recovery,
             inherit_page_attributes=inherit_page_attributes,
             access_mode=access_mode,
+            description=description,
+            closing_stream=closing_stream,
         )
-        pdf._tmp_stream = tmp_stream
+        pdf._tmp_stream = stream if allow_overwriting_input else None
         pdf._original_filename = original_filename
         return pdf
 
@@ -811,6 +909,18 @@ class Extend_ObjectMapping:
         except KeyError:
             return default
 
+    @augment_override_cpp
+    def __contains__(self, key: Name | str) -> bool:
+        if isinstance(key, Name):
+            key = str(key)
+        return _ObjectMapping._cpp__contains__(self, key)
+
+    @augment_override_cpp
+    def __getitem__(self, key: Name | str) -> Object:
+        if isinstance(key, Name):
+            key = str(key)
+        return _ObjectMapping._cpp__getitem__(self, key)
+
 
 def check_is_box(obj) -> None:
     try:
@@ -833,7 +943,12 @@ def check_is_box(obj) -> None:
 class Extend_Page:
     @property
     def mediabox(self):
-        """Return page's /MediaBox, in PDF units."""
+        """Return page's /MediaBox, in PDF units.
+
+        According to the PDF specification:
+        "The media box defines the boundaries of the physical medium on which
+        the page is to be printed."
+        """
         return self._get_mediabox(True)
 
     @mediabox.setter
@@ -841,10 +956,50 @@ class Extend_Page:
         check_is_box(value)
         self.obj['/MediaBox'] = value
 
+    @property
+    def artbox(self):
+        """Return page's effective /ArtBox, in PDF units.
+
+        According to the PDF specification:
+        "The art box defines the page's meaningful content area, including
+        white space."
+
+        If the /ArtBox is not defined, the /CropBox is returned.
+        """
+        return self._get_artbox(True, False)
+
+    @artbox.setter
+    def artbox(self, value):
+        check_is_box(value)
+        self.obj['/ArtBox'] = value
+
+    @property
+    def bleedbox(self):
+        """Return page's effective /BleedBox, in PDF units.
+
+        According to the PDF specification:
+        "The bleed box defines the region to which the contents of the page
+        should be clipped when output in a print production environment."
+
+        If the /BleedBox is not defined, the /CropBox is returned.
+        """
+        return self._get_bleedbox(True, False)
+
+    @bleedbox.setter
+    def bleedbox(self, value):
+        check_is_box(value)
+        self.obj['/BleedBox'] = value
+
     @property
     def cropbox(self):
         """Return page's effective /CropBox, in PDF units.
 
+        According to the PDF specification:
+        "The crop box defines the region to which the contents of the page
+        shall be clipped (cropped) when displayed or printed. It has no
+        defined meaning in the context of the PDF imaging model; it merely
+        imposes clipping on the page contents."
+
         If the /CropBox is not defined, the /MediaBox is returned.
         """
         return self._get_cropbox(True, False)
@@ -858,6 +1013,12 @@ class Extend_Page:
     def trimbox(self):
         """Return page's effective /TrimBox, in PDF units.
 
+        According to the PDF specification:
+        "The trim box defines the intended dimensions of the finished page
+        after trimming. It may be smaller than the media box to allow for
+        production-related content, such as printing instructions, cut marks,
+        or color bars."
+
         If the /TrimBox is not defined, the /CropBox is returned (and if
         /CropBox is not defined, /MediaBox is returned).
         """
@@ -872,15 +1033,35 @@ class Extend_Page:
     def images(self) -> _ObjectMapping:
         """Return all regular images associated with this page.
 
-        This method does not recurse into Form XObjects and does not
-        attempt to find inline images.
+        This method does not search for Form XObjects that contain images,
+        and does not attempt to find inline images.
         """
         return self._images
 
+    @property
+    def form_xobjects(self) -> _ObjectMapping:
+        """Return all Form XObjects associated with this page.
+
+        This method does not recurse into nested Form XObjects.
+
+        .. versionadded:: 7.0.0
+        """
+        return self._form_xobjects
+
     @property
     def resources(self) -> Dictionary:
-        """Return this page's resources dictionary."""
-        return self.obj['/Resources']
+        """Return this page's resources dictionary.
+
+        .. versionchanged:: 7.0.0
+            If the resources dictionary does not exist, an empty one will be created.
+            A TypeError is raised if a page has a /Resources key but it is not a
+            dictionary.
+        """
+        if Name.Resources not in self.obj:
+            self.obj.Resources = Dictionary()
+        elif not isinstance(self.obj.Resources, Dictionary):
+            raise TypeError("Page /Resources exists but is not a dictionary")
+        return self.obj.Resources
 
     def add_resource(
         self,
@@ -923,12 +1104,7 @@ class Extend_Page:
             Returns the name of the overlay in the resources dictionary instead
             of returning None.
         """
-        if Name.Resources not in self.obj:
-            self.obj.Resources = Dictionary()
-        elif not isinstance(self.obj.Resources, Dictionary):
-            raise TypeError("Page /Resources exists but is not a dictionary")
-        resources = self.obj.Resources
-
+        resources = self.resources
         if res_type not in resources:
             resources[res_type] = Dictionary()
 
@@ -1202,13 +1378,19 @@ class Extend_Attachments(MutableMapping):
         yield from self._get_all_filespecs()
 
     def __repr__(self):
-        return f"<pikepdf._qpdf.Attachments with {len(self)} attached files>"
+        return f"<pikepdf._core.Attachments with {len(self)} attached files>"
 
 
 @augments(AttachedFileSpec)
 class Extend_AttachedFileSpec:
     @staticmethod
-    def from_filepath(pdf: Pdf, path: Path | str, *, description: str = ''):
+    def from_filepath(
+        pdf: Pdf,
+        path: Path | str,
+        *,
+        description: str = '',
+        relationship: Name | None = Name.Unspecified,
+    ):
         """Construct a file specification from a file path.
 
         This function will automatically add a creation and modified date
@@ -1222,6 +1404,11 @@ class Extend_AttachedFileSpec:
             path: A file path for the file to attach to this Pdf.
             description: An optional description. May be shown to the user in
                 PDF viewers.
+            relationship: An optional relationship type. May be used to
+                indicate the type of attachment, e.g. Name.Source or Name.Data.
+                Canonically, this should be a name from the PDF specification:
+                Source, Data, Alternative, Supplement, EncryptedPayload, FormData,
+                Schema, Unspecified. If omitted, Unspecified is used.
         """
         mime, _ = mimetypes.guess_type(str(path))
         if mime is None:
@@ -1240,15 +1427,27 @@ class Extend_AttachedFileSpec:
                 datetime.datetime.fromtimestamp(stat.st_ctime)
             ),
             mod_date=encode_pdf_date(datetime.datetime.fromtimestamp(stat.st_mtime)),
+            relationship=relationship,
         )
 
+    @property
+    def relationship(self) -> Name | None:
+        return self.obj.get(Name.AFRelationship)
+
+    @relationship.setter
+    def relationship(self, value: Name | None):
+        if value is None:
+            del self.obj[Name.AFRelationship]
+        else:
+            self.obj[Name.AFRelationship] = value
+
     def __repr__(self):
         if self.filename:
             return (
-                f"<pikepdf._qpdf.AttachedFileSpec for {self.filename!r}, "
+                f"<pikepdf._core.AttachedFileSpec for {self.filename!r}, "
                 f"description {self.description!r}>"
             )
-        return f"<pikepdf._qpdf.AttachedFileSpec description {self.description!r}>"
+        return f"<pikepdf._core.AttachedFileSpec description {self.description!r}>"
 
 
 @augments(AttachedFile)
@@ -1278,7 +1477,7 @@ class Extend_AttachedFile:
 
     def __repr__(self):
         return (
-            f'<pikepdf._qpdf.AttachedFile objid={self.obj.objgen} size={self.size} '
+            f'<pikepdf._core.AttachedFile objid={self.obj.objgen} size={self.size} '
             f'mime_type={self.mime_type} creation_date={self.creation_date} '
             f'mod_date={self.mod_date}>'
         )
