@@ -4,10 +4,10 @@
 from __future__ import annotations
 
 import pytest
-from hypothesis import given
-from hypothesis.strategies import binary, sampled_from
+from conftest import skip_if_pypy
 
-from pikepdf import DataDecodingError, Name, Pdf, PdfError, Stream, _qpdf
+import pikepdf
+from pikepdf import DataDecodingError, DeletedObjectError, Name, Pdf, Stream, _core
 
 
 @pytest.fixture
@@ -25,7 +25,7 @@ def test_foreign_linearization(vera):
 
 @pytest.mark.parametrize('msg, expected', [('QPDF', 'pikepdf.Pdf')])
 def test_translate_qpdf_logic_error(msg, expected):
-    assert _qpdf._translate_qpdf_logic_error(msg) == expected
+    assert _core._translate_qpdf_logic_error(msg) == expected
 
 
 @pytest.mark.parametrize(
@@ -42,3 +42,19 @@ def test_data_decoding_errors(filter_: str, data: bytes, msg: str):
     st = Stream(p, data, Filter=Name(filter_))
     with pytest.raises(DataDecodingError, match=msg):
         st.read_bytes()
+
+
+def test_system_error():
+    with pytest.raises(FileNotFoundError):
+        pikepdf._core._test_file_not_found()
+
+
+@skip_if_pypy
+def test_return_object_from_closed():
+    p = Pdf.new()
+    obj = p.Root.TestObject = p.make_stream(b'test stream')
+    p.close()
+    del p
+    assert repr(obj) != ''
+    with pytest.raises(DeletedObjectError):
+        obj.read_bytes()
