@@ -3,7 +3,6 @@
 
 from __future__ import annotations
 
-from datetime import timedelta
 from itertools import repeat
 
 import pytest
@@ -19,7 +18,7 @@ from pikepdf import (
     Pdf,
     make_page_destination,
 )
-from pikepdf.models.outlines import ALL_PAGE_LOCATION_KWARGS, Outline
+from pikepdf.models.outlines import ALL_PAGE_LOCATION_KWARGS
 
 # pylint: disable=redefined-outer-name
 
@@ -392,7 +391,7 @@ def test_dest_or_action(outlines_doc):
     assert '/Dest' not in first_obj
 
 
-@settings(deadline=timedelta(milliseconds=750))
+@settings(deadline=60000)
 @given(
     page_num=st.integers(0, 1),
     page_loc=st.sampled_from(list(PageLocation) + ['invalid']),  # type: ignore
@@ -440,9 +439,9 @@ def test_page_destination(resources, page_num, page_loc, kwargs):
         assert dest == expected_dest
 
 
-@settings(deadline=timedelta(milliseconds=750))
+@settings(deadline=60000)
 @given(
-    title=st.text(),
+    title=st.text(max_size=1000),
     page_num=st.integers(0, 1),
     page_loc=st.sampled_from(PageLocation),
 )
@@ -451,6 +450,11 @@ def test_page_destination(resources, page_num, page_loc, kwargs):
     page_num=0,
     page_loc='FitR',
 )
+@example(
+    title='þÿ',
+    page_num=0,
+    page_loc=PageLocation.XYZ,
+)
 def test_new_item(resources, title, page_num, page_loc):
     # @given precludes use of outlines_doc fixture - causes hypothesis health check to
     # fail
