@@ -4,7 +4,6 @@
 from __future__ import annotations
 
 import shutil
-import sys
 from subprocess import PIPE, run
 
 import pytest
@@ -14,19 +13,19 @@ from pikepdf import (
     ContentStreamInlineImage,
     ContentStreamInstruction,
     Dictionary,
+    Matrix,
     Name,
     Object,
     Operator,
     Pdf,
     PdfError,
     PdfInlineImage,
-    PdfMatrix,
     Stream,
-    _qpdf,
+    _core,
     parse_content_stream,
     unparse_content_stream,
 )
-from pikepdf._qpdf import StreamParser
+from pikepdf._core import StreamParser
 from pikepdf.models import PdfParsingError
 
 # pylint: disable=useless-super-delegation,redefined-outer-name
@@ -66,7 +65,7 @@ class ExceptionParser(StreamParser):
 
 def slow_unparse_content_stream(instructions):
     def encode(obj):
-        return _qpdf.unparse(obj)
+        return _core.unparse(obj)
 
     def encode_iimage(iimage: PdfInlineImage):
         return iimage.unparse()
@@ -157,10 +156,10 @@ def test_invalid_stream_object():
     with pytest.raises(TypeError, match="called on page or stream"):
         parse_content_stream(Dictionary({"/Hi": 3}))
 
+    false_page = Dictionary(Type=Name.Page, Contents=42)
     with pytest.raises(
         TypeError, match="parse_content_stream called on non-stream Object"
     ):
-        false_page = Dictionary(Type=Name.Page, Contents=42)
         parse_content_stream(false_page)
 
 
@@ -184,7 +183,7 @@ def test_invalid_stream_object():
 def test_unparse_cs():
     instructions = [
         ([], Operator('q')),
-        ([*PdfMatrix.identity().shorthand], Operator('cm')),
+        (Matrix().as_array(), Operator('cm')),
         ([], Operator('Q')),
     ]
     assert unparse_content_stream(instructions).strip() == b'q\n1 0 0 1 0 0 cm\nQ'
@@ -208,7 +207,7 @@ def test_parse_results(inline):
     cmds = parse_content_stream(p0)
     assert isinstance(cmds[0], ContentStreamInstruction)
     csi = cmds[0]
-    assert isinstance(csi.operands, _qpdf._ObjectList)
+    assert isinstance(csi.operands, _core._ObjectList)
     assert isinstance(csi.operator, Operator)
     assert 'Operator' in repr(csi)
 
@@ -274,7 +273,7 @@ def test_end_inline_parse():
         q 200 0 0 200 500 500 cm
         BI
         /W 1
-        /H 1 
+        /H 1
         /BPC 8
         /CS /RGB
         ID \x80\x80\x80
